﻿using Connection.Class;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using WBP_O3.Class;

namespace WBP_O3_API_Demo
{
    /// <summary>
    /// MainWindow.xaml 的互動邏輯
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            this.Closing += Form_Closing;

            InitializeComponent(); 
            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            var WBP_O3Info = System.Reflection.Assembly.GetAssembly(typeof(WBP_O3.WBP_O3)).GetName();
            Title = $"{assembly.GetName().Name}  v{assembly.GetName().Version.ToString()}           {WBP_O3Info.Name}  v{WBP_O3Info.Version}";
        }
        private static WBP_O3.WBP_O3 o3;
        string key = "";
        private void Form1_Load(object sender, EventArgs e)
        {
            comboBox1.SelectedIndex = 0;
            comboBox3.SelectedIndex = 1;
        }
        private void Form_Closing(object sender, CancelEventArgs e)
        {
        }
        private void O3_OnDisConnected()
        {
            try
            {
                this.Dispatcher.Invoke(() =>
                {
                    this.Label_DevicePath.Content = "";
                    button_Send.IsEnabled = false;
                });
            }
            catch { }
        }

        private void O3_OnConnected()
        {
            this.Dispatcher.Invoke(() =>
            {
                this.Label_DevicePath.Content = o3.DevicePath;
                button_Send.IsEnabled = true;
            });
        }

        private void Button_Send_Click(object sender, EventArgs e)
        {
            Command callback;
            StringBuilder sb = new StringBuilder();
            switch (comboBox1.SelectedIndex)
            {
                case 0:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.dataReadAll());
                    if (callback == null)
                    {
                        richTextBox1.Document.Blocks.Clear();
                        richTextBox1.Document.Blocks.Add(new Paragraph(new Run("No call back,Please try again")));
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    comboBox2.Items.Clear();
                    List<MData> listMData = (List<MData>)callback.Data;
                    listMData.Where(x => x.Condition == 1 && x.CSBP > 0).ToList().ForEach(x => comboBox2.Items.Add(x.Index));
                    try
                    {
                        comboBox2.SelectedIndex = 0;
                    }
                    catch { }
                    PrintAllProperties(listMData);
                    break;
                case 1:
                    if (comboBox2.SelectedIndex == -1)
                    {
                        richTextBox1.SetText("Select a id in combobox first.");
                        return;
                    }
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.dataRead((int)comboBox2.SelectedItem, (WBP_O3.Enum.DataFormat)(comboBox3.SelectedIndex == 2 ? 3 : comboBox3.SelectedIndex)));
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD ==(byte) WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    Data data = (Data)callback.Data;
                    PrintAllProperties(data);
                    break;
                case 2:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.DataClear());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    richTextBox1.SetText("Memory have clear!");
                    break;
                case 3:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.userInfoRead());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    UserInfo userInfo = (UserInfo)callback.Data;
                    sb.AppendLine(String.Format("UserID:{0}", userInfo.UserID));
                    sb.AppendLine(String.Format("Protocal ID:{0}", userInfo.ProtocolID));
                    sb.AppendLine(String.Format("Max memory:{0}", userInfo.MaxMemory));
                    sb.AppendLine(String.Format("IHB:{0}", userInfo.IHB));
                    sb.AppendLine(String.Format("Firewall version:{0}", userInfo.FMVersion));
                    sb.AppendLine(String.Format("Firewall Date", userInfo.FMDate.ToString("yyyy-MM-dd")));
                    sb.AppendLine(String.Format("CBP:{0}", userInfo.CBP));
                    sb.AppendLine(String.Format("Voltage:{0} V", userInfo.BatteryVoltage));
                    sb.AppendLine(String.Format("arrhythmiaDisplay:{0}", userInfo.arrhythmiaDisplay.ToString()));
                    sb.AppendLine(String.Format("AFIB:{0}", userInfo.Afib));
                    richTextBox1.SetText(sb.ToString());
                    textBox_NewID.Text = userInfo.UserID;
                    break;
                case 4:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.userIDWrite(textBox_NewID.Text));
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    if (callback.CMD == (byte)WBP_O3.Enum.Command.userIDWrite)
                        richTextBox1.SetText("New ID have writed");
                    break;
                case 5:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingRead_ABPM());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    Setting_ABPM setting_ABPM = (Setting_ABPM)callback.Data;
                    PrintAllProperties(setting_ABPM);
                    textBox1.Text = setting_ABPM.ABPMStart_1.ToString();
                    textBox2.Text = setting_ABPM.ABPMEnd_1.ToString();
                    textBox3.Text = setting_ABPM.ABPMInt_1.ToString();
                    textBox4.Text = setting_ABPM.ABPMInt_2.ToString();
                    comboBox4.SelectedIndex = setting_ABPM.CBP_zone1_meas_off ? 1 : 0;
                    comboBox5.SelectedIndex = setting_ABPM.CBP_zone2_meas_off ? 1 : 0;
                    textBox5.Text = setting_ABPM.CBPInt_1.ToString();
                    textBox6.Text = setting_ABPM.CBPInt_2.ToString();
                    textBox7.Text = setting_ABPM.HI_infPressure.ToString();
                    comboBox6.SelectedIndex = setting_ABPM.SW_checkhide ? 1 : 0;
                    comboBox7.SelectedIndex = setting_ABPM.SW_SEL_silent ? 1 : 0;
                    break;
                case 6:
                    Setting_ABPM Newsetting_ABPM = new Setting_ABPM()
                    {
                        ABPMStart_1 = int.Parse(textBox1.Text),
                        ABPMEnd_1 = int.Parse(textBox2.Text),
                        ABPMInt_1 = int.Parse(textBox3.Text),
                        ABPMInt_2 = int.Parse(textBox4.Text),
                        CBP_zone1_meas_off = comboBox4.SelectedIndex == 1,
                        CBP_zone2_meas_off = comboBox5.SelectedIndex == 1,
                        CBPInt_1 = int.Parse(textBox5.Text),
                        CBPInt_2 = int.Parse(textBox6.Text),
                        HI_infPressure = int.Parse(textBox7.Text),
                        SW_checkhide = comboBox6.SelectedIndex == 1,
                        SW_SEL_silent = comboBox7.SelectedIndex == 1,
                    };
                    richTextBox1.SetText("ABPM setting had been set to:\r\n");
                    PrintAllProperties(Newsetting_ABPM, true);
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingWrite_ABPM(Newsetting_ABPM));
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    if (callback.CMD == (byte)WBP_O3.Enum.Command.settingWrite_ABPM)
                        richTextBox1.SetText("Write Success!");
                    break;
                case 7:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.deviceIDRead());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    DeviceID_and_Info deviceID_And_Info = (DeviceID_and_Info)callback.Data;
                    PrintAllProperties(deviceID_And_Info);
                    break;
                case 8:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.timeRead());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    DateTime dateTime = (DateTime)callback.Data;
                    richTextBox1.SetText(dateTime.ToString("yyyy-MM-dd HH:mm:ss"));
                    break;
                case 9:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.timeWrite(DateTime.Now));
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    if (callback.CMD == (byte)WBP_O3.Enum.Command.timeWrite)
                        richTextBox1.SetText("Time had been sync");
                    break;
                case 10:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingRead_BPM_Function());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    Setting_BPM_Function setting_BPM = (Setting_BPM_Function)callback.Data;
                    PrintAllProperties(setting_BPM);
                    break;
                case 11:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.bluetoothnameRead());
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    string s = (string)callback.Data;
                    richTextBox1.SetText(s);
                    break;
                case 12:
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.devicDecrypt(textBox8.Text));
                    if (callback == null)
                    {
                        richTextBox1.SetText("No call back,Please try again");
                        return;
                    }
                    else if (callback.CMD == (byte)WBP_O3.Enum.Command.NACK)
                    {
                        richTextBox1.SetText("NACK");
                        return;
                    }
                    richTextBox1.SetText("ACK");
                    break;
            }
        }


        int 縮排 = 0;
        private string IntToSpace(int n)
        {
            return string.Join("", Enumerable.Repeat(' ', n));
        }
        readonly StringBuilder sb = new StringBuilder();

        private void PrintAllProperties(object obj, bool append = false)
        {
            sb.Clear();
            if (obj is IList)
            {
                foreach (var _ in obj as IList)
                {
                    PrintAllPropertiesToTemp(_, true);
                    sb.AppendLine();
                }
            }
            else
            {
                PrintAllPropertiesToTemp(obj, true);
            }
            if (!append)
                richTextBox1.Clear();
            richTextBox1.AppendText(sb.ToString());
        }
        private void PrintAllPropertiesToTemp(object obj, bool append = false)
        {
            foreach (PropertyDescriptor descriptor in TypeDescriptor.GetProperties(obj))
            {
                string name = descriptor.Name;
                object value = descriptor.GetValue(obj);

                bool printed = false;
                int index = 0;
                if (value is List<int>)
                {
                    sb.AppendFormat(IntToSpace(縮排) + "{0}:", name);
                    ((List<int>)value).ToList().ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    printed = true;
                }
                else if (value is List<double>)
                {
                    sb.AppendFormat(IntToSpace(縮排) + "{0}:", name);
                    ((List<double>)value).ToList().ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    printed = true;
                }
                else if (value is List<double>[])
                {
                    sb.AppendFormat(IntToSpace(縮排) + "{0}_R:", name);
                    ((List<double>[])value)[0].ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    sb.AppendLine();
                    sb.AppendFormat(IntToSpace(縮排) + "{0}_L:", name);
                    ((List<double>[])value)[1].ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    sb.AppendLine();
                    printed = true;
                }
                else if (value is List<string>)
                {
                    sb.AppendFormat(IntToSpace(縮排) + "{0}:{1}\r\n", name, string.Join(",", (List<string>)value));
                    printed = true;
                }
                else if (value is int[])
                {
                    sb.AppendFormat(IntToSpace(縮排) + "{0}:", name);
                    ((int[])value).ToList().ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    sb.AppendLine();
                    printed = true;
                }
                else if (value is List<int>[])
                {
                    sb.AppendFormat(IntToSpace(縮排) + "{0}_R:", name);
                    ((List<int>[])value)[0].ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    sb.AppendLine();
                    sb.AppendFormat(IntToSpace(縮排) + "{0}_L:", name);
                    ((List<int>[])value)[1].ForEach(x => sb.AppendFormat("{0}{1}", index++ == 0 ? "" : ",", x));
                    sb.AppendLine();
                    printed = true;
                }
                else if (value is DateTimeOffset)
                {
                    sb.AppendLine(IntToSpace(縮排) + $"{name}: {((DateTimeOffset)value).ToString("yyyy-MM-dd HH:mm:ss")}");
                    printed = true;
                }
                else if (value is string)
                {
                    sb.AppendLine(IntToSpace(縮排) + String.Format("{0}:{1}", name, value));
                    printed = true;
                }
                if (value is IList && !printed)
                {
                    richTextBox1.AppendText(IntToSpace(縮排) + string.Format("{0}:\r\n", name));
                    縮排 += 2;
                    foreach (var temp in value as IList)
                    {
                        PrintAllProperties(temp, true);
                        richTextBox1.AppendText(Environment.NewLine);
                    }
                    縮排 -= 2;
                    printed = true;
                }
                else if (descriptor.PropertyType.IsClass && !printed)
                {
                    richTextBox1.AppendText(IntToSpace(縮排) + string.Format("{0}:\r\n", name));
                    縮排 += 2;
                    PrintAllProperties(value, true);
                    縮排 -= 2;
                    printed = true;
                }
                if (!printed)
                    sb.AppendLine(IntToSpace(縮排) + String.Format("{0}:{1}", name, value));
            }
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            try
            {
                key = textBox_key.Text;
                o3 = new WBP_O3.WBP_O3(key);
                o3.OnConnected += O3_OnConnected;
                o3.OnDisConnected += O3_OnDisConnected;
                button_BLE.IsEnabled = true;
                textBox_key.IsEnabled = false;
                (sender as Button).IsEnabled = false;
            }
            catch (Exception ee)
            {
                MessageBox.Show(ee.Message);
            }
        }
        [STAThread]
        private async void Button_BLE_Click(object sender, EventArgs e)
        {
            if (!Connection.GlobalFunction.WinRT_API_Support)
            {
                MessageBox.Show("Operating system requires Windows 10 1803 or higher!");
                return;
            }
            Window_BluetoothDiscover form_BLE_Discover = new Window_BluetoothDiscover();

            form_BLE_Discover.ShowDialog();
            switch (form_BLE_Discover.BLEAction)
            {
                case BLEAction.Connect:
                    o3?.Dispose();
                    o3 = new WBP_O3.WBP_O3(key, true);

                    o3.OnDisConnected += O3_OnDisConnected;
                    var connectionResult = await o3.ConnectWithBLE(form_BLE_Discover.BLE_ID);
                    if (connectionResult.ErrorMessage == "Device is unreachable (i.e. out of range or shutoff)")
                    {
                        MessageBox.Show("Please check the connection or unpair BLE, both of PC and blood pressure monitor.");
                        O3_OnDisConnected();
                    }
                    if (o3.Connected)
                    {
                        await Connection.PairingHelper.PairDeviceAsync(form_BLE_Discover.BLE_ID);
                        await Task.Delay(500);
                        try
                        {
                            await o3.SendMessageAsync(WBP_O3.SendMessage.ToCommand.timeRead());
                        }
                        catch (Exception ee)
                        {
                            if (ee.Message.Contains("0x80650005"))
                            {
                                //PC unpair BLE
                                await Connection.PairingHelper.UnpairDeviceAsync(form_BLE_Discover.BLE_ID);
                                MessageBox.Show("Please unpair BLE on blood pressure monitor.");
                                O3_OnDisConnected();
                                return;
                            }
                        }
                        O3_OnConnected();
                    }
                    break;
                case BLEAction.Disconnect:
                    //await o3.SendMessageAsync(WBP_o3.SendMessage.ToCommand.bluetoothDisconnect());
                    if (o3.ConnectionType == 1)
                    {
                        o3?.Dispose();
                        o3 = new WBP_O3.WBP_O3(key);
                        o3.OnConnected += O3_OnConnected;
                        o3.OnDisConnected += O3_OnDisConnected;
                    }
                    break;
                case BLEAction.Pair:
                    await Connection.PairingHelper.PairDeviceAsync(form_BLE_Discover.BLE_ID);
                    break;
                case BLEAction.Unpair:
                    await Connection.PairingHelper.UnpairDeviceAsync(form_BLE_Discover.BLE_ID);
                    break;
            }

        }
    }
}
